/* Copyright (C) 2011-2013 Research In Motion Limited.  */

#ifndef cascades_imageviewloadeffect_h
#define cascades_imageviewloadeffect_h

#include <bb/cascades/bbcascades_global.h>
#include <QObject>

namespace bb {
    namespace cascades {

        /*!
         * 
         * @brief Load effects for images in an @c ImageView.
         *
         * These load effects are used to determine how an image is animated while it's being
         * loaded into an @c ImageView.
         *
         * @xmlonly
         * <apigrouping group="User interface/Core controls"/>
         * @endxmlonly
         *
         * @since BlackBerry 10.0.0
         */

        class BBCASCADES_EXPORT ImageViewLoadEffect {
            Q_GADGET
            Q_ENUMS (Type)
        public:
        
            /*!
             * @brief Controls how an image is visualized after initial loading.
             *
             * @since BlackBerry 10.0.0
             */
            enum Type {
            
                /*! 
                 * @brief A default animation which is determined by type of image (asset or non-asset) and
                 * the context of how it's loaded into a control.
                 *
                 * An image that is bundled with the application as an asset is loaded immediately. Image assets also block
                 * the UI, which means that the image assets are displayed simulataneously with other controls .
                 * These types of images are displayed without any particular effect (@c None).
                 * 
                 * For images that aren't assets (for example, generated by the app or retrieved from a remote source),
                 * the framework attempts to load these images before the UI needs to be displayed.
                 * If loading completes in time, the image appears immediately. If the image doesn't load in time,
                 * the image is loaded asynchronously so that it doesn't block the rest of the UI from being displayed.
                 *
                 * @since BlackBerry 10.0.0
                 */
                Default = 0,
                
                /*!
                 * @brief Same as @c Default, but assets are also subject to deferred loading.
                 *
                 * @since BlackBerry 10.0.0
                 */
                DefaultDeferred = 1,
                
                /*! @brief No animation is used.
                 *
                 * This setting displays the image without any particular effect. If used with an asset,
                 * the image blocks the UI from being displayed until the image finishes loading. Images that
                 * aren't assets are loaded asynchronously, and don't block the UI.
                 *
                 * @since BlackBerry 10.0.0
                 */
                None = 2,
                /*!
                 * @brief The image animates in with a subtle, non-intrusive effect.
                 *
                 * @since BlackBerry 10.0.0
                 */
                Subtle = 3,
                /*!
                 * @brief The image fades in with a zoom animation that scales the image up to
                 * its full size.
                 *
                 * @since BlackBerry 10.0.0
                 */
                FadeZoom = 4

            };

        private:
            /*! \cond PRIVATE */
            ImageViewLoadEffect();
            ~ImageViewLoadEffect();
            /*! \endcond */
        };
    }
}


#endif // cascades_imageviewloadeffect_h
